// This file contains the 'main' function. Program execution begins and ends there.//

#include <iostream>
#include <libusb.h>
#include <ctime>																								`Q



#define VID 0x0641				
#define PID 0x1DC3
constexpr auto OUT_ENPOINT = 0x01;
constexpr auto IN_ENPOINT = 0x81;

using namespace std;
libusb_device** devs;				//pointer to pointer of device, used to retrieve a list of devices
libusb_device_handle* dev_handle;	//a device handle
libusb_context* ctx = 0;			//a libusb session
int r;								//for return values
ssize_t cnt;						//holding number of devices in list

void write_PoStep(unsigned char* data)
{
	int actual;
	int r = 0;
	cout << "Writing Command OUT_DATA[1]: ";
	printf("%X", data[1]);
	cout << endl;

	r = libusb_bulk_transfer(dev_handle, OUT_ENPOINT, data, 65, &actual, 0);
	if (r == 0 && actual == 65)						//we wrote the 65 bytes successfully
		cout << "Writing Successful!" << endl;
	else
		cout << "Write Error" << endl;
}

void read_PoStep(unsigned char* data)
{
	int actual;
	int r = 0;
	cout << "Reading Data..." << endl;
	int i = 0;

	r = libusb_interrupt_transfer(dev_handle, IN_ENPOINT, data, 64, &actual, 500);
	if (r == 0 && actual == 64)						//we read the 64 bytes successfully
	{
		cout << "Reading Successful!" << endl;
		while (i < 64)
		{
			cout << "IN_DATA[" << i << "]: ";
			printf("%X", data[i]);
			cout << endl;
			i++;
		}
	}
	else
		cout << "Read Error" << endl;
}


void waitfor(int intr)
{
	int u = 1;
	clock_t begin = clock();

	while (u)
	{
		clock_t end = clock();
		if (end - begin > intr)
		u = 0;
	}
}

int main() {

	r = libusb_init(&ctx);
	dev_handle = libusb_open_device_with_vid_pid(ctx, PID, VID); // vendorID and productID I found for my usb device
	if (dev_handle == NULL)
		cout << "Cannot open device" << endl;
	else
		cout << "Device Opened" << endl;
	libusb_free_device_list(devs, 1);							// free the list, unref the devices in it

	unsigned char* data = new unsigned char[64];				//data to write

	/*Select command data */
//	data[0] = 0x00; data[1] = 0xA0; data[63] = 0x00;						// Command Enable RTS
//	data[0] = 0x00; data[1] = 0x01; data[63] = 0x00;						// Command GET DEVICE INFO
//	data[0] = 0x00; data[1] = 0xA1; data[20] = 0x00; data[63] = 0x00;		// Command RUN/SLEEP
//  data[0] = 0x00; data[1] = 0x90; data[20] = 0xE0; data[21] = 0x2E; data[22] = 0x00; data[23] = 0x00; data[24] = 0x00; data[63] = 0x00;	//  Command STEP/DIR  
//	data[0] = 0x00; data[1] = 0x02; data[20] = 0x01; data[63] = 0x00;		//  Command SYSRST

	int actual;													//used to find out how many bytes were written

	if (libusb_kernel_driver_active(dev_handle, 0) == 1) {		//find out if kernel driver is attached
		cout << "Kernel Driver Active" << endl;
		if (libusb_detach_kernel_driver(dev_handle, 0) == 0)	//detach it
			cout << "Kernel Driver Detached!" << endl;
	}
	r = libusb_claim_interface(dev_handle, 0);					//claim interface 0 (the first) of device (mine had just 1)
	if (r < 0) {
		cout << "Cannot Claim Interface" << endl;
		return 1;
	}
	cout << "Claimed Interface" << endl;

	data[0] = 0x00; data[1] = 0x01; data[63] = 0x00;			// Command GET DEVICE INFO
	write_PoStep(data);
	read_PoStep(data);
	float d = 0;
	d = data[45] * 0.125;										// Calculate temperature
	cout << "Temperature= ";
	printf( "%f", d);
	cout << endl;
	d = ((int)data[8]*256 + data[9])*0.072;						// Calculate voltage
	cout << "Voltage= ";
	printf("%f", d);
	cout << endl;

	data[0] = 0x00; data[1] = 0xA1; data[20] = 0x00; data[63] = 0x00;		//Sleep... Command RUN/SLEEP
	write_PoStep(data);
	data[0] = 0x00; data[1] = 0x90; data[20] = 0xE0; data[21] = 0x2E; data[22] = 0x00; data[23] = 0x00; data[24] = 0x00; data[63] = 0x00; //Set 40 steps/sec... Command STEP/DIR 
	write_PoStep(data);
	data[0] = 0x00; data[1] = 0xA1; data[20] = 0x01; data[63] = 0x00;		//Run... Command RUN/SLEEP
	write_PoStep(data);
	waitfor(5000);															//wait for 5000 clocks
	data[0] = 0x00; data[1] = 0xA1; data[20] = 0x00; data[63] = 0x00;		//Sleep... Command RUN/SLEEP 
	write_PoStep(data);
	data[0] = 0x00; data[1] = 0x90; data[20] = 0x70; data[21] = 0x17; data[22] = 0x00; data[23] = 0x00; data[24] = 0x01; data[63] = 0x00; //Set 40 steps/sec invert dir...  Command STEP/DIR
	write_PoStep(data);
	cout << "Changing direction" << endl;
	data[0] = 0x00; data[1] = 0xA1; data[20] = 0x01; data[63] = 0x00;		//Run... Command RUN/SLEEP
	write_PoStep(data);
	waitfor(5000);															// wait for 5000 clocks
	data[0] = 0x00; data[1] = 0xA1; data[20] = 0x00; data[63] = 0x00;		//Sleep... Command RUN/SLEEP
	write_PoStep(data);

	r = libusb_release_interface(dev_handle, 0);	//release the claimed interface
	if (r != 0) {
		cout << "Cannot Release Interface" << endl;
		return 1;
	}
	cout << "Released Interface" << endl;

	libusb_close(dev_handle);						//close the device we opened
	libusb_exit(ctx);								//needs to be called to end the

	delete[] data;									//delete the allocated memory for data
	return 0;

}